==> ui-18.0.3.tgz: package/components/tooltip/Tooltip.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useImperativeHandle, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useEventCallback, useId, useRefExtra } from '@react-devui/hooks';
import { getClassName, getPopupPosition } from '@react-devui/utils';
import { useMaxIndex, useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { useComponentConfig, usePrefixConfig } from '../root';
const TTANSITION_DURING = { enter: 86, leave: 100 };
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTooltip' });
function Tooltip(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dTrigger = 'hover', dContainer, dPlacement = 'top', dEscClosable = true, dArrow = true, dDistance = 10, dInWindow = false, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dZIndex, dTitle, onVisibleChange, afterVisibleChange } = _b, restProps = __rest(_b, ["children", "dVisible", "dTrigger", "dContainer", "dPlacement", "dEscClosable", "dArrow", "dDistance", "dInWindow", "dMouseEnterDelay", "dMouseLeaveDelay", "dZIndex", "dTitle", "onVisibleChange", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const triggerRef = useRefExtra(() => document.querySelector(`[aria-describedby="${id}"]`));
    const popupRef = useRef(null);
    const containerRef = useRefExtra(isUndefined(dContainer)
        ? () => {
            let el = document.getElementById(`${dPrefix}tooltip-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}tooltip-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a, _b;
                return (_b = (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dContainer, true);
    //#endregion
    const uniqueId = useId();
    const id = (_a = restProps.id) !== null && _a !== void 0 ? _a : `${dPrefix}tooltip-${uniqueId}`;
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (isUndefined(dZIndex)) {
            if (isUndefined(dContainer)) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return dZIndex;
        }
    })();
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [placement, setPlacement] = useState(dPlacement);
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (visible && popupRef.current && triggerRef.current) {
            let currentPlacement = dPlacement;
            let space = [0, 0, 0, 0];
            if (!isUndefined(dContainer) && containerRef.current) {
                const containerRect = containerRef.current.getBoundingClientRect();
                space = [
                    containerRect.top,
                    window.innerWidth - containerRect.left - containerRect.width,
                    window.innerHeight - containerRect.top - containerRect.height,
                    containerRect.left,
                ];
            }
            const position = getPopupPosition(popupRef.current, triggerRef.current, {
                placement: dPlacement,
                offset: dDistance,
                inWindow: dInWindow,
            }, space);
            if (position) {
                currentPlacement = position.placement;
                setPlacement(position.placement);
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            else {
                const position = getPopupPosition(popupRef.current, triggerRef.current, {
                    placement,
                    offset: dDistance,
                    inWindow: dInWindow,
                });
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            let transformOrigin = 'center bottom';
            switch (currentPlacement) {
                case 'top':
                    transformOrigin = 'center bottom';
                    break;
                case 'top-left':
                    transformOrigin = '20px bottom';
                    break;
                case 'top-right':
                    transformOrigin = 'calc(100% - 20px) bottom';
                    break;
                case 'right':
                    transformOrigin = 'left center';
                    break;
                case 'right-top':
                    transformOrigin = 'left 12px';
                    break;
                case 'right-bottom':
                    transformOrigin = 'left calc(100% - 12px)';
                    break;
                case 'bottom':
                    transformOrigin = 'center top';
                    break;
                case 'bottom-left':
                    transformOrigin = '20px top';
                    break;
                case 'bottom-right':
                    transformOrigin = 'calc(100% - 20px) top';
                    break;
                case 'left':
                    transformOrigin = 'right center';
                    break;
                case 'left-top':
                    transformOrigin = 'right 12px';
                    break;
                case 'left-bottom':
                    transformOrigin = 'right calc(100% - 12px)';
                    break;
                default:
                    break;
            }
            setTransformOrigin(transformOrigin);
        }
    });
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    return (_jsx(DPopup, Object.assign({ dVisible: visible, dTrigger: dTrigger, dEscClosable: dEscClosable, dMouseEnterDelay: dMouseEnterDelay, dMouseLeaveDelay: dMouseLeaveDelay, dUpdatePosition: {
            fn: updatePosition,
            triggerRef,
            popupRef,
            extraScrollRefs: [isUndefined(dContainer) ? undefined : containerRef],
        }, onVisibleChange: changeVisible }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(React.cloneElement(children, {
                    'aria-describedby': id,
                })), containerRef.current &&
                    ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING, onEnter: updatePosition, afterEnter: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                        }, afterLeave: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                        } }, { children: (state) => {
                            let transitionStyle = {};
                            switch (state) {
                                case 'enter':
                                    transitionStyle = { transform: 'scale(0.3)', opacity: 0 };
                                    break;
                                case 'entering':
                                    transitionStyle = {
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.enter}ms ease-out`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaving':
                                    transitionStyle = {
                                        transform: 'scale(0.3)',
                                        opacity: 0,
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.leave}ms ease-in`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaved':
                                    transitionStyle = { display: 'none' };
                                    break;
                                default:
                                    break;
                            }
                            return renderPopup(_jsxs("div", Object.assign({}, restProps, { ref: popupRef, id: id, className: getClassName(restProps.className, `${dPrefix}tooltip`, `${dPrefix}tooltip--` + placement), style: Object.assign(Object.assign(Object.assign(Object.assign({}, restProps.style), popupPositionStyle), { zIndex }), transitionStyle), role: "tooltip" }, { children: [dArrow && _jsx("div", { className: `${dPrefix}tooltip__arrow` }), dTitle] })));
                        } })), containerRef.current)] })) })));
}
export const DTooltip = React.forwardRef(Tooltip);

==> ui-18.0.5.tgz: package/components/tooltip/Tooltip.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useImperativeHandle, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import { useEvent, useEventCallback, useId, useRefExtra } from '@react-devui/hooks';
import { getClassName, getPopupPosition } from '@react-devui/utils';
import { useMaxIndex, useDValue } from '../../hooks';
import { checkNoExpandedEl, registerComponentMate } from '../../utils';
import { EXPANDED_DATA } from '../../utils/checkNoExpandedEl';
import { DPopup } from '../_popup';
import { DTransition } from '../_transition';
import { useComponentConfig, usePrefixConfig } from '../root';
const TTANSITION_DURING = { enter: 86, leave: 100 };
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTooltip' });
function Tooltip(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { children, dVisible, dTrigger = 'hover', dContainer, dPlacement = 'top', dEscClosable = true, dArrow = true, dDistance = 10, dInWindow = false, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dZIndex, dTitle, onVisibleChange, afterVisibleChange } = _b, restProps = __rest(_b, ["children", "dVisible", "dTrigger", "dContainer", "dPlacement", "dEscClosable", "dArrow", "dDistance", "dInWindow", "dMouseEnterDelay", "dMouseLeaveDelay", "dZIndex", "dTitle", "onVisibleChange", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const windowRef = useRefExtra(() => window);
    const triggerRef = useRefExtra(() => document.querySelector(`[aria-describedby="${id}"]`));
    const popupRef = useRef(null);
    const containerRef = useRefExtra(isUndefined(dContainer)
        ? () => {
            let el = document.getElementById(`${dPrefix}tooltip-root`);
            if (!el) {
                el = document.createElement('div');
                el.id = `${dPrefix}tooltip-root`;
                document.body.appendChild(el);
            }
            return el;
        }
        : dContainer === false
            ? () => {
                var _a, _b;
                return (_b = (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dContainer, true);
    //#endregion
    const uniqueId = useId();
    const id = (_a = restProps.id) !== null && _a !== void 0 ? _a : `${dPrefix}tooltip-${uniqueId}`;
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const maxZIndex = useMaxIndex(visible);
    const zIndex = (() => {
        if (isUndefined(dZIndex)) {
            if (isUndefined(dContainer)) {
                return maxZIndex;
            }
            else {
                return `var(--${dPrefix}zindex-absolute)`;
            }
        }
        else {
            return dZIndex;
        }
    })();
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: '-200vh',
        left: '-200vw',
    });
    const [placement, setPlacement] = useState(dPlacement);
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        if (visible && popupRef.current && triggerRef.current) {
            let currentPlacement = dPlacement;
            let space = [0, 0, 0, 0];
            if (!isUndefined(dContainer) && containerRef.current) {
                const containerRect = containerRef.current.getBoundingClientRect();
                space = [
                    containerRect.top,
                    window.innerWidth - containerRect.left - containerRect.width,
                    window.innerHeight - containerRect.top - containerRect.height,
                    containerRect.left,
                ];
            }
            const position = getPopupPosition(popupRef.current, triggerRef.current, {
                placement: dPlacement,
                offset: dDistance,
                inWindow: dInWindow,
            }, space);
            if (position) {
                currentPlacement = position.placement;
                setPlacement(position.placement);
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            else {
                const position = getPopupPosition(popupRef.current, triggerRef.current, {
                    placement,
                    offset: dDistance,
                    inWindow: dInWindow,
                });
                setPopupPositionStyle({
                    top: position.top,
                    left: position.left,
                });
            }
            let transformOrigin = 'center bottom';
            switch (currentPlacement) {
                case 'top':
                    transformOrigin = 'center bottom';
                    break;
                case 'top-left':
                    transformOrigin = '20px bottom';
                    break;
                case 'top-right':
                    transformOrigin = 'calc(100% - 20px) bottom';
                    break;
                case 'right':
                    transformOrigin = 'left center';
                    break;
                case 'right-top':
                    transformOrigin = 'left 12px';
                    break;
                case 'right-bottom':
                    transformOrigin = 'left calc(100% - 12px)';
                    break;
                case 'bottom':
                    transformOrigin = 'center top';
                    break;
                case 'bottom-left':
                    transformOrigin = '20px top';
                    break;
                case 'bottom-right':
                    transformOrigin = 'calc(100% - 20px) top';
                    break;
                case 'left':
                    transformOrigin = 'right center';
                    break;
                case 'left-top':
                    transformOrigin = 'right 12px';
                    break;
                case 'left-bottom':
                    transformOrigin = 'right calc(100% - 12px)';
                    break;
                default:
                    break;
            }
            setTransformOrigin(transformOrigin);
        }
    });
    useEvent(windowRef, 'keydown', (e) => {
        if (e.code === 'Escape' && popupRef.current && checkNoExpandedEl(popupRef.current)) {
            changeVisible(false);
        }
    }, {}, !dEscClosable || !visible);
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    return (_jsx(DPopup, Object.assign({ dVisible: visible, dTrigger: dTrigger, dMouseEnterDelay: dMouseEnterDelay, dMouseLeaveDelay: dMouseLeaveDelay, dUpdatePosition: {
            fn: updatePosition,
            triggerRef,
            popupRef,
            extraScrollRefs: [isUndefined(dContainer) ? undefined : containerRef],
        }, onVisibleChange: changeVisible }, { children: ({ renderTrigger, renderPopup }) => (_jsxs(_Fragment, { children: [renderTrigger(React.cloneElement(children, {
                    'aria-describedby': id,
                    [EXPANDED_DATA]: visible,
                })), containerRef.current &&
                    ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING, onEnter: updatePosition, afterEnter: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                        }, afterLeave: () => {
                            afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                        } }, { children: (state) => {
                            let transitionStyle = {};
                            switch (state) {
                                case 'enter':
                                    transitionStyle = { transform: 'scale(0.3)', opacity: 0 };
                                    break;
                                case 'entering':
                                    transitionStyle = {
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.enter}ms ease-out`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaving':
                                    transitionStyle = {
                                        transform: 'scale(0.3)',
                                        opacity: 0,
                                        transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING.leave}ms ease-in`).join(', '),
                                        transformOrigin,
                                    };
                                    break;
                                case 'leaved':
                                    transitionStyle = { display: 'none' };
                                    break;
                                default:
                                    break;
                            }
                            return renderPopup(_jsxs("div", Object.assign({}, restProps, { ref: popupRef, id: id, className: getClassName(restProps.className, `${dPrefix}tooltip`, `${dPrefix}tooltip--` + placement), style: Object.assign(Object.assign(Object.assign(Object.assign({}, restProps.style), popupPositionStyle), { zIndex }), transitionStyle), role: "tooltip" }, { children: [dArrow && _jsx("div", { className: `${dPrefix}tooltip__arrow` }), dTitle] })));
                        } })), containerRef.current)] })) })));
}
export const DTooltip = React.forwardRef(Tooltip);
